/*******************************
 *          __                   
 *         /\ \                  
 *  __  _  \_\ \     __    ___   
 * /\ \/'\ /'_' \  /'__'\ /'___\ 
 * \/>  <//\ \ \ \/\  __//\ \__/ 
 *  /\_/\_\ \___,_\ \____\ \____\
 *  \//\/_/\/__,_ /\/____/\/____/
 * 
 *  Author: declis (xdec.de)
 ********************************/ 

#include <msp430g2452.h>
#include "lib_lcd.h"
#include "delay.h"

#define cycle		326				// 100Hz, T=10ms
#define delay_ms	5				// delay in ms
#define adc_res		1023			// 10bit ADC
const char bright[]={"Brightness:"};

void main(void)
{	
	unsigned long duty_cycle=0;
	unsigned int adc_old=0;		
		 
	WDTCTL=WDTPW+WDTHOLD;			// stop WDT
	BCSCTL1=CALBC1_1MHZ;        	// set DCO to 1MHz, set range
  	DCOCTL=CALDCO_1MHZ;				// set DCO step+modulation
	init_USI();						// initialize USI
	init_LCD();						// initialize LCD (DOGM162)
	
	P1DIR|=BIT2;					// P1.2 -> output
	P1SEL|=BIT2;					// P1.2 -> compare: Out1 output (TA0.1)
	TACCR0=cycle;					// CCR0 -> cycle time (10ms)
	TACCR1=0;						// CCR1 -> duty cycle
	TACCTL1=OUTMOD_7;				// reset/set
	TACTL=TASSEL_1+MC_1;			// CLK-source=32kHz, up-mode
	
	BCSCTL3|=XCAP_3;				// CL=~12.5pF
	do
	{
		IFG1&=~OFIFG;
		wait_ms(1);
	}
	while(IFG1&OFIFG);
	
	//Channel 0 (Poti); ADC10 CLK divider = /4; CLK=ADC10OSC
  	ADC10CTL1=INCH_0+ADC10DIV_3+ADC10SSEL_0;
  	//Vref+=Vcc (ext. LDO 3.3V) and Vss=Vr-; S&H time 16x; ADC10 on         
  	ADC10CTL0=SREF_0+ADC10SHT_3+ADC10ON;  
  	// P1.0 is analog input
  	ADC10AE0|=BIT0;    
	
	write_string(0,0,bright);
	write_char(14,0,'%');
	while(1)
	{
		wait_ms(delay_ms);
		ADC10CTL0|=ENC+ADC10SC;					// start & enable conversion		
		while(ADC10CTL0&ADC10BUSY);				// ADC busy? 
		
		// if we don't have a new ADC value, do nothing and wait
		if(adc_old!=ADC10MEM)
		{
			// y=mx+b
			duty_cycle=((cycle*(unsigned long)ADC10MEM)/adc_res);	
			TACCR1=duty_cycle;	
			adc_old=ADC10MEM;												
		}
		// duty cycle in percent -> duty-cycle=(cycle/100)*percent
		char_to_num_out(12,0,(duty_cycle*100)/cycle);
	}
}
